@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -24,6 +24,48 @@
 #include "disk.h"
 #include "api_metadata.h"
 #include "io/reader.h"
+#include "vfs/filesystem.h"
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Check if object is an instance of <i>mobius.disk.disk</i>
+//! \param pyobj Python object
+//! \return true/false
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+bool
+pymobius_disk_disk_check (PyObject *pyobj)
+{
+  return PyObject_IsInstance (pyobj, (PyObject *) &disk_disk_t);
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Create <b>disk</b> Python object from C++ object
+//! \param d Disk object
+//! \return Python object
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+PyObject *
+pymobius_disk_disk_to_pyobject (mobius::disk::disk d)
+{
+  PyObject *ret = _PyObject_New (&disk_disk_t);
+
+  if (ret)
+    ((disk_disk_o *) ret)->obj = new mobius::disk::disk (d);
+
+  return ret;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Create <i>mobius::disk::disk</i> C++ object from Python object
+//! \param value Python object
+//! \return C++ object
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+mobius::disk::disk
+pymobius_disk_disk_from_pyobject (PyObject *value)
+{
+  if (!pymobius_disk_disk_check (value))
+    throw std::invalid_argument (mobius::MOBIUS_EXCEPTION_MSG ("object must be an instance of mobius.disk.disk"));
+
+  return * (reinterpret_cast <disk_disk_o *>(value)->obj);
+}
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief tp_new (default constructor)
@@ -50,7 +92,7 @@ static void
 tp_dealloc (disk_disk_o *self)
 {
   delete self->obj;
-  self->ob_type->tp_free ((PyObject*) self);
+  Py_TYPE (self)->tp_free ((PyObject*) self);
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -59,7 +101,7 @@ tp_dealloc (disk_disk_o *self)
 static PyObject *
 tp_getter_size (disk_disk_o *self)
 {
-  return PyLong_FromUnsignedLongLong (self->obj->get_size ());
+  return mobius::py::pylong_from_std_uint64_t (self->obj->get_size ());
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -68,7 +110,7 @@ tp_getter_size (disk_disk_o *self)
 static PyObject *
 tp_getter_sector_size (disk_disk_o *self)
 {
-  return PyLong_FromUnsignedLong (self->obj->get_sector_size ());
+  return mobius::py::pylong_from_std_uint32_t (self->obj->get_sector_size ());
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -77,7 +119,7 @@ tp_getter_sector_size (disk_disk_o *self)
 static PyObject *
 tp_getter_name (disk_disk_o *self)
 {
-  return PyString2_from_std_string (self->obj->get_name ());
+  return mobius::py::pystring_from_std_string (self->obj->get_name ());
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -86,7 +128,7 @@ tp_getter_name (disk_disk_o *self)
 static PyObject *
 tp_getter_type (disk_disk_o *self)
 {
-  return PyString2_from_std_string (self->obj->get_type ());
+  return mobius::py::pystring_from_std_string (self->obj->get_type ());
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -127,7 +169,18 @@ static PyGetSetDef tp_getset[] =
 static PyObject *
 tp_f_is_available (disk_disk_o *self, PyObject *)
 {
-  return mobius::py::pybool_from_bool (self->obj->is_available ());
+  PyObject *ret = nullptr;
+
+  try
+    {
+      ret = mobius::py::pybool_from_bool (self->obj->is_available ());
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_runtime_error (e.what ());
+    }
+
+  return ret;
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -136,7 +189,18 @@ tp_f_is_available (disk_disk_o *self, PyObject *)
 static PyObject *
 tp_f_new_reader (disk_disk_o *self, PyObject *)
 {
-  return PyMobius_IO_Reader_from_cpp (self->obj->new_reader ());
+  PyObject *ret = nullptr;
+
+  try
+    {
+     ret = pymobius_io_reader_to_pyobject (self->obj->new_reader ());
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_runtime_error (e.what ());
+    }
+
+  return ret;
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -145,10 +209,19 @@ tp_f_new_reader (disk_disk_o *self, PyObject *)
 static PyObject *
 tp_f_scan (disk_disk_o *self, PyObject *)
 {
-  self->obj->scan ();
+  PyObject *ret = nullptr;
 
-  Py_INCREF (Py_None);
-  return Py_None;
+  try
+    {
+      self->obj->scan ();
+      ret = mobius::py::pynone ();
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_runtime_error (e.what ());
+    }
+
+  return ret;
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -157,7 +230,40 @@ tp_f_scan (disk_disk_o *self, PyObject *)
 static PyObject *
 tp_f_get_metadata (disk_disk_o *self, PyObject *)
 {
-  return PyList_FromMetadata (self->obj->get_metadata ());
+  PyObject *ret = nullptr;
+
+  try
+    {
+      ret = PyList_FromMetadata (self->obj->get_metadata ());
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_runtime_error (e.what ());
+    }
+
+  return ret;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief get_filesystems
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_get_filesystems (disk_disk_o *self, PyObject *)
+{
+  PyObject *ret = nullptr;
+
+  try
+    {
+      ret = mobius::py::pylist_from_cpp_container (
+               self->obj->get_filesystems (), pymobius_vfs_filesystem_to_pyobject
+            );
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_runtime_error (e.what ());
+    }
+
+  return ret;
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -189,6 +295,12 @@ static PyMethodDef tp_methods[] =
     METH_VARARGS,
     "get metadata"
   },
+  {
+    (char *) "get_filesystems",
+    (PyCFunction) tp_f_get_filesystems,
+    METH_VARARGS,
+    "get filesystems"
+  },
   {NULL, NULL, 0, NULL} // sentinel
 };
 
@@ -244,4 +356,5 @@ PyTypeObject disk_disk_t =
   0,                                       		// tp_weaklist
   0,                                       		// tp_del
   0,                                       		// tp_version_tag
+  0,                                       		// tp_finalize
 };
