@@ -21,8 +21,12 @@
 //! \author Eduardo Aguiar
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 #include <pymobius.h>
+#include <pycallback.h>
+#include <pyobject.h>
+#include <functional>
 #include "item.h"
 #include "module.h"
+#include "account.h"
 #include "ant.h"
 #include "application.h"
 #include "bookmarked_url.h"
@@ -31,6 +35,7 @@
 #include "chat_message.h"
 #include "cookie.h"
 #include "encryption_key.h"
+#include "ip_address.h"
 #include "opened_file.h"
 #include "password.h"
 #include "password_hash.h"
@@ -41,6 +46,7 @@
 #include "visited_url.h"
 #include "pod/data.h"
 #include "pod/list.h"
+#include <mobius/crypt/hash.h>
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Check if object is an instance of <i>mobius.model.item</i>
@@ -804,6 +810,41 @@ tp_f_get_ants (model_item_o *self, PyObject *)
   return ret;
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief <i>count_evidences</i> method implementation
+//! \param self Object
+//! \param args Argument list
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_count_evidences (model_item_o *self, PyObject *args)
+{
+  // parse input args
+  std::string arg_type;
+
+  try
+    {
+      arg_type = mobius::py::get_arg_as_std_string (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_invalid_type_error (e.what ());
+      return nullptr;
+    }
+
+  PyObject *ret = nullptr;
+
+  try
+    {
+      ret = mobius::py::pylong_from_std_int64_t (self->obj->count_evidences (arg_type));
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_runtime_error (e.what ());
+    }
+
+  return ret;
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief <i>new_password</i> method implementation
 //! \param self Object
@@ -2264,6 +2305,204 @@ tp_f_count_encryption_keys (model_item_o *self, PyObject *)
   return ret;
 }
 
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief <i>new_ip_address</i> method implementation
+//! \param self Object
+//! \param args Argument list
+//! \param New Python object
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_new_ip_address (model_item_o *self, PyObject *args)
+{
+  // Parse input args
+  mobius::datetime::datetime arg_timestamp;
+  std::string arg_address;
+
+  try
+    {
+      arg_timestamp = mobius::py::get_arg_as_datetime (args, 0);
+      arg_address = mobius::py::get_arg_as_std_string (args, 1);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
+
+  // Execute C++ function
+  PyObject *ret = nullptr;
+
+  try
+    {
+      ret = pymobius_model_ip_address_to_pyobject (
+              self->obj->new_ip_address (arg_timestamp, arg_address)
+            );
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_Exception, e.what ());
+    }
+
+  return ret;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief <i>get_ip_addresses</i> method implementation
+//! \param self Object
+//! \return List of objects
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_get_ip_addresses (model_item_o *self, PyObject *)
+{
+  PyObject *ret = nullptr;
+
+  try
+    {
+      ret = mobius::py::pylist_from_cpp_container (
+              self->obj->get_ip_addresses (),
+              pymobius_model_ip_address_to_pyobject
+            );
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_Exception, e.what ());
+    }
+
+  return ret;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief <i>remove_ip_addresses</i> method implementation
+//! \param self Object
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_remove_ip_addresses (model_item_o *self, PyObject *)
+{
+  try
+    {
+      self->obj->remove_ip_addresses ();
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_Exception, e.what ());
+      return nullptr;
+    }
+
+  return mobius::py::pynone ();
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief <i>count_ip_addresses</i> method implementation
+//! \param self Object
+//! \return Number of items
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_count_ip_addresses (model_item_o *self, PyObject *)
+{
+  PyObject *ret = nullptr;
+
+  try
+    {
+      ret = mobius::py::pylong_from_std_int64_t (
+              self->obj->count_ip_addresses ()
+            );
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_Exception, e.what ());
+    }
+
+  return ret;
+}
+
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief <i>new_account</i> method implementation
+//! \param self Object
+//! \param args Argument list
+//! \param New Python object
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_new_account (model_item_o *self, PyObject *args)
+{
+  // Parse input args
+  std::string arg_type;
+  std::string arg_id;
+
+  try
+    {
+      arg_type = mobius::py::get_arg_as_std_string (args, 0);
+      arg_id = mobius::py::get_arg_as_std_string (args, 1);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
+
+  // Execute C++ function
+  PyObject *ret = nullptr;
+
+  try
+    {
+      ret = pymobius_model_account_to_pyobject (
+              self->obj->new_account (arg_type, arg_id)
+            );
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_Exception, e.what ());
+    }
+
+  return ret;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief <i>get_accounts</i> method implementation
+//! \param self Object
+//! \return List of objects
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_get_accounts (model_item_o *self, PyObject *)
+{
+  PyObject *ret = nullptr;
+
+  try
+    {
+      ret = mobius::py::pylist_from_cpp_container (
+              self->obj->get_accounts (),
+              pymobius_model_account_to_pyobject
+            );
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_Exception, e.what ());
+    }
+
+  return ret;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief <i>remove_accounts</i> method implementation
+//! \param self Object
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_remove_accounts (model_item_o *self, PyObject *)
+{
+  try
+    {
+      self->obj->remove_accounts ();
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_Exception, e.what ());
+      return nullptr;
+    }
+
+  return mobius::py::pynone ();
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Methods structure
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -2383,6 +2622,12 @@ static PyMethodDef tp_methods[] =
     METH_VARARGS,
     "Get ANTs"
   },
+  {
+    (char *) "count_evidences",
+    (PyCFunction) tp_f_count_evidences,
+    METH_VARARGS,
+    "Count evidences by type"
+  },
   {
     (char *) "new_password",
     (PyCFunction) tp_f_new_password,
@@ -2689,6 +2934,48 @@ static PyMethodDef tp_methods[] =
     METH_VARARGS,
     "Count encryption_key objects"
   },
+  {
+    (char *) "new_ip_address",
+    (PyCFunction) tp_f_new_ip_address,
+    METH_VARARGS,
+    "Create new ip_address object"
+  },
+  {
+    (char *) "get_ip_addresses",
+    (PyCFunction) tp_f_get_ip_addresses,
+    METH_VARARGS,
+    "Get ip_address objects"
+  },
+  {
+    (char *) "remove_ip_addresses",
+    (PyCFunction) tp_f_remove_ip_addresses,
+    METH_VARARGS,
+    "Remove ip_address objects"
+  },
+  {
+    (char *) "count_ip_addresses",
+    (PyCFunction) tp_f_count_ip_addresses,
+    METH_VARARGS,
+    "Count ip_address objects"
+  },
+  {
+    (char *) "new_account",
+    (PyCFunction) tp_f_new_account,
+    METH_VARARGS,
+    "Create new account object"
+  },
+  {
+    (char *) "get_accounts",
+    (PyCFunction) tp_f_get_accounts,
+    METH_VARARGS,
+    "Get account objects"
+  },
+  {
+    (char *) "remove_accounts",
+    (PyCFunction) tp_f_remove_accounts,
+    METH_VARARGS,
+    "Remove account objects"
+  },
   {NULL, NULL, 0, NULL} // sentinel
 };
 
@@ -2815,6 +3102,76 @@ tp_setattro (PyObject *o, PyObject *name, PyObject *value)
   return 0;
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief <i>item</i> tp_richcompare
+//! \param py_a Item Object
+//! \param py_b Item Object
+//! \param op Operation
+//! \return Either Py_True or Py_False
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_richcompare (PyObject *py_a, PyObject *py_b, int op)
+{
+  PyObject *ret = nullptr;
+
+  if (!pymobius_model_item_check (py_a) || !pymobius_model_item_check (py_b))
+    ret = mobius::py::py_false ();
+
+  else
+    {
+      auto a = * (reinterpret_cast <model_item_o *> (py_a)->obj);
+      auto b = * (reinterpret_cast <model_item_o *> (py_b)->obj);
+      bool rc = false;
+
+      switch (op)
+        {
+        case Py_EQ:
+          rc = (a == b);
+          break;
+
+        case Py_NE:
+          rc = (a != b);
+          break;
+
+        case Py_LT:
+          rc = (a < b);
+          break;
+
+        case Py_LE:
+          rc = (a <= b);
+          break;
+
+        case Py_GT:
+          rc = (a > b);
+          break;
+
+        case Py_GE:
+          rc = (a >= b);
+          break;
+        }
+
+      ret = rc ? mobius::py::py_true () : mobius::py::py_false ();
+    }
+
+  return ret;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief <i>item</i> tp_hash
+//! \param self Item Object
+//! \return Item hash
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static Py_hash_t
+tp_hash (model_item_o *self)
+{
+  // As negative value indicates error, return non-negative value
+  return std::abs (
+    static_cast <Py_hash_t> (
+       std::hash <mobius::model::item> {} (*(self->obj))
+    )
+  );
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Type structure
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -2833,7 +3190,7 @@ PyTypeObject model_item_t =
   0,                                       		// tp_as_number
   0,                                       		// tp_as_sequence
   0,                                       		// tp_as_mapping
-  0,                                       		// tp_hash
+  reinterpret_cast <hashfunc> (tp_hash),		// tp_hash
   0,                                       		// tp_call
   0,                                       		// tp_str
   tp_getattro,                                 		// tp_getattro
@@ -2843,7 +3200,7 @@ PyTypeObject model_item_t =
   "item class",                            		// tp_doc
   0,                                       		// tp_traverse
   0,                                       		// tp_clear
-  0,                                       		// tp_richcompare
+  tp_richcompare,                             		// tp_richcompare
   0,                                       		// tp_weaklistoffset
   0,                                       		// tp_iter
   0,                                       		// tp_iternext
@@ -2869,3 +3226,69 @@ PyTypeObject model_item_t =
   0,                                       		// tp_version_tag
   0,                                       		// tp_finalize
 };
+
+namespace
+{
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief attribute-modified event callback
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+class attribute_modified_callback
+{
+public:
+  attribute_modified_callback (PyObject *f)
+    : f_ (f)
+  {
+  }
+
+  void
+  operator () (
+    const mobius::model::item& item,
+    const std::string& id,
+    const mobius::pod::data& old_value,
+    const mobius::pod::data& new_value)
+  {
+    f_.call (
+      pymobius_model_item_to_pyobject (item),
+      mobius::py::pystring_from_std_string (id),
+      pymobius_pod_data_to_pyobject (old_value),
+      pymobius_pod_data_to_pyobject (new_value)
+    );
+  }
+
+private:
+  mobius::py::pyobject f_;
+};
+
+mobius::py::callback <attribute_modified_callback> cb_1_ ("attribute-modified");
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief attribute-removed event callback
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+class attribute_removed_callback
+{
+public:
+  attribute_removed_callback (PyObject *f)
+    : f_ (f)
+  {
+  }
+
+  void
+  operator () (
+    const mobius::model::item& item,
+    const std::string& id,
+    const mobius::pod::data& old_value)
+  {
+    f_.call (
+      pymobius_model_item_to_pyobject (item),
+      mobius::py::pystring_from_std_string (id),
+      pymobius_pod_data_to_pyobject (old_value)
+    );
+  }
+
+private:
+  mobius::py::pyobject f_;
+};
+
+mobius::py::callback <attribute_removed_callback> cb_2_ ("attribute-removed");
+
+} // namespace
