@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -31,7 +31,7 @@
 //! \return true/false
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 bool
-PyMobius_OS_Win_Registry_Registry_Key_Check (PyObject *value)
+pymobius_os_win_registry_registry_key_check (PyObject *value)
 {
   return PyObject_IsInstance (value, reinterpret_cast<PyObject *> (&os_win_registry_registry_key_t));
 }
@@ -42,7 +42,7 @@ PyMobius_OS_Win_Registry_Registry_Key_Check (PyObject *value)
 //! \return Python object
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 PyObject *
-PyMobius_OS_Win_Registry_Registry_Key_from_cpp (mobius::os::win::registry::registry_key r)
+pymobius_os_win_registry_registry_key_to_pyobject (mobius::os::win::registry::registry_key r)
 {
   PyObject *ret = nullptr;
 
@@ -65,9 +65,9 @@ PyMobius_OS_Win_Registry_Registry_Key_from_cpp (mobius::os::win::registry::regis
 //! \return C++ object
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 mobius::os::win::registry::registry_key
-PyMobius_OS_Win_Registry_Registry_Key_as_cpp (PyObject *pyobj)
+pymobius_os_win_registry_registry_key_from_pyobject (PyObject *pyobj)
 {
-  if (!PyMobius_OS_Win_Registry_Registry_Key_Check (pyobj))
+  if (!pymobius_os_win_registry_registry_key_check (pyobj))
     throw std::invalid_argument (mobius::MOBIUS_EXCEPTION_MSG ("object must be an instance of mobius.os.win.registry_key"));
 
   return * (reinterpret_cast <os_win_registry_registry_key_o *>(pyobj)->obj);
@@ -80,7 +80,7 @@ static void
 tp_dealloc (os_win_registry_registry_key_o *self)
 {
   delete self->obj;
-  self->ob_type->tp_free ((PyObject*) self);
+  Py_TYPE (self)->tp_free ((PyObject*) self);
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -93,11 +93,11 @@ tp_getter_name (os_win_registry_registry_key_o *self)
 
   try
     {
-      ret = PyString2_from_std_string (self->obj->get_name ());
+      ret = mobius::py::pystring_from_std_string (self->obj->get_name ());
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -112,14 +112,14 @@ tp_setter_name (os_win_registry_registry_key_o *self, PyObject *value, void *)
   // if it is a delete operation, returns with error
   if (value == nullptr)
     {
-      PyErr_SetString (PyExc_TypeError, "cannot delete 'name' attribute");
+      mobius::py::set_invalid_type_error ("cannot delete 'name' attribute");
       return -1;
     }
 
   // check argument type
   if (!mobius::py::pystring_check (value))
     {
-      PyErr_SetString (PyExc_TypeError, "invalid type for 'name' attribute");
+      mobius::py::set_invalid_type_error ("invalid type for 'name' attribute");
       return -1;
     }
 
@@ -130,7 +130,7 @@ tp_setter_name (os_win_registry_registry_key_o *self, PyObject *value, void *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return ok
@@ -151,7 +151,7 @@ tp_getter_classname (os_win_registry_registry_key_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -171,7 +171,7 @@ tp_getter_last_modification_time (os_win_registry_registry_key_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -189,12 +189,12 @@ tp_getter_subkeys (os_win_registry_registry_key_o *self)
     {
       ret = mobius::py::pylist_from_cpp_container (
                *(self->obj),
-               PyMobius_OS_Win_Registry_Registry_Key_from_cpp
+               pymobius_os_win_registry_registry_key_to_pyobject
             );
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -212,12 +212,12 @@ tp_getter_values (os_win_registry_registry_key_o *self)
     {
       ret = mobius::py::pylist_from_cpp_container (
                self->obj->get_values (),
-               PyMobius_OS_Win_Registry_Registry_Value_from_cpp
+               pymobius_os_win_registry_registry_value_to_pyobject
             );
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -277,11 +277,11 @@ tp_f_add_key (os_win_registry_registry_key_o *self, PyObject *args)
 
   try
     {
-      arg_key = mobius::py::get_arg_as_cpp (args, 0, PyMobius_OS_Win_Registry_Registry_Key_as_cpp);
+      arg_key = mobius::py::get_arg_as_cpp (args, 0, pymobius_os_win_registry_registry_key_from_pyobject);
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_TypeError, e.what ());
+      mobius::py::set_invalid_type_error (e.what ());
       return nullptr;
     }
 
@@ -295,7 +295,7 @@ tp_f_add_key (os_win_registry_registry_key_o *self, PyObject *args)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -316,7 +316,7 @@ tp_f_remove_key (os_win_registry_registry_key_o *self, PyObject *args)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_TypeError, e.what ());
+      mobius::py::set_invalid_type_error (e.what ());
       return nullptr;
     }
 
@@ -330,7 +330,7 @@ tp_f_remove_key (os_win_registry_registry_key_o *self, PyObject *args)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -352,7 +352,7 @@ tp_f_clear_keys (os_win_registry_registry_key_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -373,7 +373,7 @@ tp_f_get_key_by_path (os_win_registry_registry_key_o *self, PyObject *args)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_TypeError, e.what ());
+      mobius::py::set_invalid_type_error (e.what ());
       return nullptr;
     }
 
@@ -382,11 +382,11 @@ tp_f_get_key_by_path (os_win_registry_registry_key_o *self, PyObject *args)
 
   try
     {
-      ret = PyMobius_OS_Win_Registry_Registry_Key_from_cpp (self->obj->get_key_by_path (arg_path));
+      ret = pymobius_os_win_registry_registry_key_to_pyobject (self->obj->get_key_by_path (arg_path));
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -408,7 +408,7 @@ tp_f_get_key_by_name (os_win_registry_registry_key_o *self, PyObject *args)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_TypeError, e.what ());
+      mobius::py::set_invalid_type_error (e.what ());
       return nullptr;
     }
 
@@ -417,11 +417,11 @@ tp_f_get_key_by_name (os_win_registry_registry_key_o *self, PyObject *args)
 
   try
     {
-      ret = PyMobius_OS_Win_Registry_Registry_Key_from_cpp (self->obj->get_key_by_name (arg_name));
+      ret = pymobius_os_win_registry_registry_key_to_pyobject (self->obj->get_key_by_name (arg_name));
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -443,7 +443,7 @@ tp_f_get_key_by_mask (os_win_registry_registry_key_o *self, PyObject *args)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_TypeError, e.what ());
+      mobius::py::set_invalid_type_error (e.what ());
       return nullptr;
     }
 
@@ -454,12 +454,12 @@ tp_f_get_key_by_mask (os_win_registry_registry_key_o *self, PyObject *args)
     {
       ret = mobius::py::pylist_from_cpp_container (
                self->obj->get_key_by_mask (arg_mask),
-               PyMobius_OS_Win_Registry_Registry_Key_from_cpp
+               pymobius_os_win_registry_registry_key_to_pyobject
             );
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -480,7 +480,7 @@ tp_f_get_value_by_path (os_win_registry_registry_key_o *self, PyObject *args)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_TypeError, e.what ());
+      mobius::py::set_invalid_type_error (e.what ());
       return nullptr;
     }
 
@@ -489,11 +489,11 @@ tp_f_get_value_by_path (os_win_registry_registry_key_o *self, PyObject *args)
 
   try
     {
-      ret = PyMobius_OS_Win_Registry_Registry_Value_from_cpp (self->obj->get_value_by_path (arg_path));
+      ret = pymobius_os_win_registry_registry_value_to_pyobject (self->obj->get_value_by_path (arg_path));
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -515,7 +515,7 @@ tp_f_get_value_by_name (os_win_registry_registry_key_o *self, PyObject *args)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_TypeError, e.what ());
+      mobius::py::set_invalid_type_error (e.what ());
       return nullptr;
     }
 
@@ -524,11 +524,11 @@ tp_f_get_value_by_name (os_win_registry_registry_key_o *self, PyObject *args)
 
   try
     {
-      ret = PyMobius_OS_Win_Registry_Registry_Value_from_cpp (self->obj->get_value_by_name (arg_name));
+      ret = pymobius_os_win_registry_registry_value_to_pyobject (self->obj->get_value_by_name (arg_name));
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -550,7 +550,7 @@ tp_f_get_value_by_mask (os_win_registry_registry_key_o *self, PyObject *args)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_TypeError, e.what ());
+      mobius::py::set_invalid_type_error (e.what ());
       return nullptr;
     }
 
@@ -561,12 +561,12 @@ tp_f_get_value_by_mask (os_win_registry_registry_key_o *self, PyObject *args)
     {
       ret = mobius::py::pylist_from_cpp_container (
                self->obj->get_value_by_mask (arg_mask),
-               PyMobius_OS_Win_Registry_Registry_Value_from_cpp
+               pymobius_os_win_registry_registry_value_to_pyobject
             );
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -587,7 +587,7 @@ tp_f_get_data_by_path (os_win_registry_registry_key_o *self, PyObject *args)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_TypeError, e.what ());
+      mobius::py::set_invalid_type_error (e.what ());
       return nullptr;
     }
 
@@ -596,11 +596,11 @@ tp_f_get_data_by_path (os_win_registry_registry_key_o *self, PyObject *args)
 
   try
     {
-      ret = PyMobius_OS_Win_Registry_Registry_Data_from_cpp (self->obj->get_data_by_path (arg_path));
+      ret = pymobius_os_win_registry_registry_data_to_pyobject (self->obj->get_data_by_path (arg_path));
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -622,7 +622,7 @@ tp_f_get_data_by_name (os_win_registry_registry_key_o *self, PyObject *args)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_TypeError, e.what ());
+      mobius::py::set_invalid_type_error (e.what ());
       return nullptr;
     }
 
@@ -631,11 +631,11 @@ tp_f_get_data_by_name (os_win_registry_registry_key_o *self, PyObject *args)
 
   try
     {
-      ret = PyMobius_OS_Win_Registry_Registry_Data_from_cpp (self->obj->get_data_by_name (arg_name));
+      ret = pymobius_os_win_registry_registry_data_to_pyobject (self->obj->get_data_by_name (arg_name));
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -657,7 +657,7 @@ tp_f_get_data_by_mask (os_win_registry_registry_key_o *self, PyObject *args)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_TypeError, e.what ());
+      mobius::py::set_invalid_type_error (e.what ());
       return nullptr;
     }
 
@@ -668,12 +668,12 @@ tp_f_get_data_by_mask (os_win_registry_registry_key_o *self, PyObject *args)
     {
       ret = mobius::py::pylist_from_cpp_container (
                self->obj->get_data_by_mask (arg_mask),
-               PyMobius_OS_Win_Registry_Registry_Data_from_cpp
+               pymobius_os_win_registry_registry_data_to_pyobject
             );
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -694,7 +694,7 @@ tp_f_has_subkeys (os_win_registry_registry_key_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -839,4 +839,5 @@ PyTypeObject os_win_registry_registry_key_t =
   0,                                       		// tp_weaklist
   0,                                       		// tp_del
   0,                                       		// tp_version_tag
+  0,                                       		// tp_finalize
 };
