@@ -1,6 +1,6 @@
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 # Mobius Forensic Toolkit
-# Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020 Eduardo Aguiar
+# Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Eduardo Aguiar
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the
@@ -26,21 +26,22 @@ import pymobius.p2p.shareaza.decoder_sd
 import pymobius.app.emule
 import pymobius.operating_system
 
-import decoder_ac_searchstrings_dat
-import decoder_cancelled_met
-import decoder_ip_to_country
-import decoder_key_index_dat
-import decoder_known_met
-import decoder_part_met
-import decoder_part_met_txtsrc
-import decoder_preferences_dat
-import decoder_preferences_ini
-import decoder_preferenceskad_dat
-import decoder_stored_searches_met
-import CTag
-
 import mobius
-import urllib
+import urllib.parse
+import traceback
+
+from . import decoder_ac_searchstrings_dat
+from . import decoder_cancelled_met
+from . import decoder_ip_to_country
+from . import decoder_key_index_dat
+from . import decoder_known_met
+from . import decoder_part_met
+from . import decoder_part_met_txtsrc
+from . import decoder_preferences_dat
+from . import decoder_preferences_ini
+from . import decoder_preferenceskad_dat
+from . import decoder_stored_searches_met
+from . import CTag
 
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 # Versions examined: Emule 0.50a and DreaMule 3.2
@@ -88,22 +89,6 @@ import urllib
 class dataholder (object):
   pass
 
-# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-# @brief iterate through user folders
-# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-def iter_users_folders (vfs):
-  for root in vfs:
-    users_folder = root.get_child_by_name ('Users', False)
-
-    if not users_folder:
-      users_folder = root.get_child_by_name ('Documents and Settings', False)
-
-    if users_folder:
-      for child in users_folder.get_children ():
-        if child.is_folder ():
-          username = child.name
-          yield username, child
-
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 # @brief Retrieves Emule P2P activity data
 # @author Eduardo Aguiar
@@ -114,8 +99,8 @@ def retrieve (model):
   try:
     ant = Ant (model)
     ant.run ()
-  except Exception, e:
-    mobius.core.log ('WRN p2p.emule: %s' % str (e))
+  except Exception as e:
+    mobius.core.logf ('WRN ' + str (e) + ' ' + traceback.format_exc ())
 
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 # @brief forensics: P2P Emule
@@ -171,37 +156,54 @@ class Ant (object):
     # get item VFS
     vfs = pymobius.vfs.get_item_vfs (self.__item)
     if not vfs:
-      raise Exception, "Datasource is not available"
+      raise Exception ("Datasource is not available")
 
-    # decode ":/Program Files/Emule/Config"
+    # decode "<USER_FOLDER>/"
+    path_list = []
+
+    for opsys in pymobius.operating_system.scan (self.__item):
+      for user_profile in opsys.get_profiles ():
+        try:
+          username = user_profile.username
+          self.__incoming_dir = None
+          self.__temp_dir = None
+
+          # AppData/Local
+          folder = user_profile.get_entry_by_path ('%localappdata%/eMule')
+          if folder:
+            self.__retrieve_appdatalocal (username, folder)
+
+          # Incoming dir
+          if self.__incoming_dir:
+            path_list.append ((username, self.__incoming_dir[3:].replace ('\\', '/')))
+
+          else:
+            folder = user_profile.get_entry_by_path ('%downloads%/eMule/Incoming')
+            if folder:
+              path_list.append ((username, folder.path[3:].replace ('\\', '/')))
+
+          # Temp dir
+          if self.__temp_dir:
+            path_list.append ((username, self.__temp_dir[3:].replace ('\\', '/')))
+
+          else:
+            folder = user_profile.get_entry_by_path ('%downloads%/eMule/Temp')
+            if folder:
+              path_list.append ((username, folder.path[3:].replace ('\\', '/')))
+
+        except Exception as e:
+          mobius.core.logf ('WRN %s %s' % (str (e), traceback.format_exc ()))
+
+    # decode root folder
     for root in vfs:
+
+      # /Program Files/Emule/Config"
       folder = root.get_child_by_path ('Program Files/Emule/Config', False)
       if folder:
         self.__retrieve_program_files_folder (folder)
 
-    # decode "<USER_FOLDER>/"
-    for username, user_folder in iter_users_folders (vfs):
-      self.__incoming_dir = None
-      self.__temp_dir = None
-
-      folder = \
-         user_folder.get_child_by_path ('AppData/Local/eMule', False) or \
-         user_folder.get_child_by_path ('Application Data/eMule', False)
-
-      if folder:
-        self.__retrieve_appdatalocal (username, folder)
-
-      # Incoming dir
-      if self.__incoming_dir:
-        path = self.__incoming_dir[3:].replace ('\\', '/')
-        folder = root.get_child_by_path (path, False)
-
-        if folder:
-          self.__retrieve_part_control_files (folder, username)
-
-      # Temp dir
-      if self.__temp_dir:
-        path = self.__temp_dir[3:].replace ('\\', '/')
+      # .part folder
+      for username, path in path_list:
         folder = root.get_child_by_path (path, False)
 
         if folder:
@@ -216,7 +218,7 @@ class Ant (object):
       fname = f.name.lower ()
 
       # Ignore reallocated entries
-      if f.is_reallocated:
+      if f.is_reallocated ():
         pass
 
       # ip-to-country.csv
@@ -231,8 +233,8 @@ class Ant (object):
 
     # config folder
     config_folder = folder.get_child_by_name ('config', False)
-    if config_folder:
-    
+    if config_folder and config_folder.is_folder ():
+
       # retrieve 'preferences.ini' first
       f = config_folder.get_child_by_name ('preferences.ini', False)
       self.__retrieve_preferences_ini (self.__item, self.__data, f, username)
@@ -250,7 +252,7 @@ class Ant (object):
       fname = f.name.lower ()
 
       # Ignore reallocated entries
-      if f.is_reallocated:
+      if f.is_reallocated ():
         pass
 
       # AC_SearchStrings.dat
@@ -320,11 +322,11 @@ class Ant (object):
 
       # StoredSearches.met
       elif fname == 'storedsearches.met':
-        self.__retrieve_stored_searches_met (self.__item, self.__data, f, username)	      
+        self.__retrieve_stored_searches_met (self.__item, self.__data, f, username)	
 
       else:
         t = 'file' if f.is_file () else 'folder'
-        mobius.core.log ('p2p.emule: Unhandled Config/%s %s' % (f.name, t))
+        mobius.core.logf ('p2p.emule: Unhandled Config/%s %s' % (f.name, t))
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   # @brief Retrieve data from .part.* control files
@@ -376,7 +378,7 @@ class Ant (object):
 
       elif path.endswith ('.part.stats'):
         pymobius.p2p.set_handled (self.__item, f)
-        
+
       elif path.endswith ('.sd'):	# Shareaza control file
         pymobius.p2p.shareaza.decoder_sd.retrieve (self.__item, self.__data, f, None)
 
@@ -477,7 +479,7 @@ class Ant (object):
     data = decoder_preferences_dat.decode (f)
     if not data:
       return
-  
+
     # create account
     account = pymobius.p2p.account.account ()
     account.username = username
@@ -496,7 +498,7 @@ class Ant (object):
   def __retrieve_preferences_ini (self, item, model, f, username):
     self.__incoming_dir = None
     self.__temp_dir = None
-    
+
     if not f:
       return
 
@@ -538,7 +540,7 @@ class Ant (object):
     data = decoder_stored_searches_met.decode (f)
     if not data:
       return
-    
+
     for csearch in data.csearches:
       search = pymobius.p2p.search.search ()
       search.app_id = 'emule'
@@ -588,7 +590,7 @@ class Ant (object):
 
     # set file handled
     pymobius.p2p.set_handled (item, f)
-    
+
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   # @brief Create new local file from CKnownFile structure
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -606,7 +608,7 @@ class Ant (object):
 
     # flags
     f.flag_shared = STATE_ALWAYS
-    f.flag_corrupted = STATE_YES if info.get ('corrupted_parts') else STATE_NO      
+    f.flag_corrupted = STATE_YES if info.get ('corrupted_parts') else STATE_NO
     f.flag_downloaded = STATE_YES
     f.flag_uploaded = STATE_YES if info.get ('uploaded_bytes') > 0 or info.get ('times_accepted', 0) > 0 else STATE_NO
     f.flag_completed = STATE_YES	# @see CPartFile::PerformFileCompleteEnd
@@ -687,19 +689,22 @@ class Ant (object):
     if self.__ip_table:
       for f in self.__data.remote_files:
         if f.peer.ip:
-          f.peer.country_code, f.peer.country = self.__ip_table.get (f.peer.ip)
+          try:
+            f.peer.country_code, f.peer.country = self.__ip_table.get (f.peer.ip)
+          except Exception as e:
+            mobius.core.logf ('WRN ' + str (e) + ' ' + traceback.format_exc ())
 
     # add URL to remote_files
     for f in self.__data.remote_files:
-      if not f.url:
-        f.url = 'ed2k://|file|%s|%d|%s|/' % (urllib.quote (f.name), f.size, f.get_hash ('ed2k'))
+      if not f.url and f.name and f.size != -1:
+        f.url = 'ed2k://|file|%s|%d|%s|/' % (urllib.parse.quote (f.name), f.size, f.get_hash ('ed2k'))
 
     # sort and remove duplicated remote files
     remote_files = [ (f.timestamp, f) for f in self.__data.remote_files ]
     timestamp, peer_ip, size, name = None, None, None, None
     self.__data.remote_files = []
 
-    for timestamp, f in sorted (remote_files):
+    for timestamp, f in sorted (remote_files, key=lambda x: x[0]):
       if (timestamp, peer_ip, size, name) != (f.timestamp, f.peer.ip, f.size, f.name):
         self.__data.remote_files.append (f)
         timestamp, peer_ip, size, name = f.timestamp, f.peer.ip, f.size, f.name
