@@ -18,7 +18,7 @@
 import mobius
 import pymobius.deprecated
 
-LAST_VFS_ITEM = None
+LAST_VFS_KEY = None
 LAST_VFS = None
 EFISYS_MAX_SIZE = 204800 * 512
 
@@ -34,11 +34,12 @@ EFISYS_MAX_SIZE = 204800 * 512
 # file descriptors and memory, it should be minimized.
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 def get_item_vfs (item):
-  global LAST_VFS_ITEM
+  global LAST_VFS_KEY
   global LAST_VFS
   v = None
+  key = (item.case.uid, item.uid)
 
-  if item.uid == LAST_VFS_ITEM and LAST_VFS.is_available ():
+  if key == LAST_VFS_KEY and LAST_VFS.is_available ():
     v = LAST_VFS
 
   else:
@@ -48,7 +49,7 @@ def get_item_vfs (item):
       if disk and disk.is_available ():
         v = vfs (disk)
         LAST_VFS = v
-        LAST_VFS_ITEM = item.uid
+        LAST_VFS_KEY = key
 
   return v
 
